@@ -6,7 +6,7 @@ from http import HTTPStatus
 from requests import Response
 from typing import Callable
 
-from flask import Flask, request, jsonify
+from flask import Flask, request, jsonify, copy_current_request_context
 
 from vantage6.node.server_io import NodeClient
 from vantage6.node.util import (
@@ -100,9 +100,7 @@ def make_request(method: str, endpoint: str, json: dict = None,
         Response from the vantage6-server
     """
 
-    # Obtain http method from request and map it to the same `requests` method
     method = get_method(method)
-    log.debug(f'method: {method}')
 
     # Forward the request to the central server. Retry when an exception is
     # raised (e.g. timeout or connection error) or when the server gives an
@@ -118,7 +116,7 @@ def make_request(method: str, endpoint: str, json: dict = None,
             if response.status_code > 210:
                 log.warn('Proxy server received status code:'
                          f'{response.status_code}')
-                log.debug(f'url: {url}, json: {json}, params: {params}, '
+                log.debug(f'method: {request.method}, url: {url}, json: {json}, params: {params}, '
                           f'headers: {headers}')
                 if 'application/json' in response.headers.get('Content-Type'):
                     log.debug(response.json().get("msg", "no description..."))
@@ -132,11 +130,10 @@ def make_request(method: str, endpoint: str, json: dict = None,
             log.exception(f'On attempt {i}, the proxy request raised an '
                           f'exception: <{url}>')
 
-        # if all attemps fail, raise an exception to be handled by its parent
+    # if all attemps fail, raise an exception to be handled by its parent
     raise Exception("Proxy request failed")
 
 
-
 def decrypt_result(result: dict) -> dict:
     """
     Decrypt the `result` from a result dictonary
@@ -188,6 +185,7 @@ def get_response_json_and_handle_exceptions(response: Response):
         log.exception('Failed to extract JSON')
     return None
 
+
 @app.route("/task", methods=["POST"])
 def proxy_task():
     """
@@ -201,6 +199,8 @@ def proxy_task():
     # We need the server io for the decryption of the results
     server_io: NodeClient = app.config.get("SERVER_IO")
     if not server_io:
+        log.error("Task proxy request received but proxy server was not "
+                  "initialized properly.")
         return jsonify({'msg': 'Proxy server not initialized properly'}), 500
 
     # All requests from algorithms are unencrypted. We encrypt the input
@@ -212,8 +212,18 @@ def proxy_task():
         log.error("No organizations found in proxy request..")
         return jsonify({"msg": "Organizations missing from input"}), 400
 
-    log.debug(f"{len(organizations)} organizations, attemping to encrypt")
+    try:
+        headers = {'Authorization': request.headers['Authorization']}
+    except Exception:
+        log.exception('Could not extract headers from request...')
 
+    log.debug(f"{len(organizations)} organizations")
+
+    # For every organization we need to encrypt the input field. This is done
+    # in parallel as the client (algorithm) is waiting for a timely response.
+    # For every organizationn the public key is retrieved an the input is
+    # encrypted specifically for them.
+    @copy_current_request_context
     def encrypt_input(organization: dict) -> dict:
         """
         Encrypt the input for a specific organization by using its private key.
@@ -235,10 +245,8 @@ def proxy_task():
 
         # retrieve public key of the organization
         log.debug(f"Retrieving public key of org: {organization_id}")
-        present = 'Authorization' in request.headers
-        headers = {'Authorization': request.headers['Authorization']} if present \
-            else None
-        response = make_request('get', f'organization/{organization_id}', headers=headers)
+        response = make_request('get', f'organization/{organization_id}',
+                                headers=headers)
         public_key = response.json().get("public_key")
 
         # Encrypt the input field
@@ -252,21 +260,16 @@ def proxy_task():
                 f"{organization_id}!")
         return organization
 
-
-    # For every organization we need to encrypt the input field. This is done
-    # in parallel as the client (algorithm) is waiting for a timely response.
-    # For every organizationn the public key is retrieved an the input is
-    # encrypted specifically for them.
     if server_io.is_encrypted_collaboration():
-        data["organizations"] = [encrypt_input(o) for o in organizations]
-        # with concurrent.futures.ThreadPoolExecutor() as executor:
-        #     futures = [executor.submit(encrypt_input, o)
-        #                for o in organizations]
-        # data["organizations"] = [future.result() for future in futures]
+        log.debug("Applying end-to-end encryption")
+
+        with concurrent.futures.ThreadPoolExecutor() as executor:
+            futures = [executor.submit(encrypt_input, o)
+                    for o in organizations]
+        data["organizations"] = [future.result() for future in futures]
 
     # Attempt to send the task to the central server
     try:
-        headers = {'Authorization': request.headers['Authorization']}
         response = make_request('post', 'task', data, headers=headers)
     except Exception:
         log.exception('post task failed')
